// termstream.cpp

#include "termstream.h"

#include <iostream>
#include <streambuf.h>
#include <ncurses.h>

int termbuf::Control::n= 0;

termbuf::Control::Control ()
{
	if (n == 0) {
		initscr ();
		noecho ();
		raw ();
		start_color ();
	}
	++n;
}

termbuf::Control::~Control ()
{
	if (--n == 0)
		endwin ();
}

otermstream::otermstream (ios::openmode which) :
	termstreambase (which)
{
	setWindow (stdscr);
}

otermstream::otermstream (int nlines, int ncols, int y, int x) :
	termstreambase (ios::out)
{
	setWindow (newwin (nlines, ncols, y, x) );
}

void otermstream::clrscr ()
{
	sync ();
	wclear ( (WINDOW *) w);
	wrefresh ( (WINDOW *) w);
}

void otermstream::clreol ()
{
	sync ();
	wclrtoeol ( (WINDOW *) w);
	wrefresh ( (WINDOW *) w);
}

void otermstream::insertline ()
{
	sync ();
	winsertln ( (WINDOW *) w);
}

void otermstream::deleteline ()
{
	sync ();
	wdeleteln ( (WINDOW *) w);
}

void otermstream::at (int y, int x)
{
	sync ();
	wmove ( (WINDOW *) w, y, x);
}

void otermstream::inverse (bool flag)
{
	sync ();
	if (flag) wattron ( (WINDOW *) w, A_REVERSE);
	else wattroff ( (WINDOW *) w, A_REVERSE);
}

void otermstream::delay (bool flag)
{
	nodelay ( (WINDOW *) w, flag ? false : true);
}

void otermstream::cursor (bool flag)
{
	curs_set (flag ? 1 : 0);
}

namespace {

short color_curses (Color::value c)
{
	short v;
	switch (c) {
	case Color::Black:   v= COLOR_BLACK;   break;
	case Color::Red:     v= COLOR_RED;     break;
	case Color::Green:   v= COLOR_GREEN;   break;
	case Color::Yellow:  v= COLOR_YELLOW;  break;
	case Color::Blue:    v= COLOR_BLUE;    break;
	case Color::Magenta: v= COLOR_MAGENTA; break;
	case Color::Cyan:    v= COLOR_CYAN;    break;
	case Color::White:   v= COLOR_WHITE;   break;
	}
	return v;
}

} // namespace

void otermstream::setcolor (short n, Color::value f, Color::value b)
{
	init_pair (n, color_curses (f), color_curses (b) );
}

void otermstream::color (short c)
{
	sync ();
	wattrset ( (WINDOW *) w, COLOR_PAIR (c) );
}

void termstreambase::setWindow (void *window)
{
	w= window;
	__my_tb.setWindow (w);
	keypad ( (WINDOW *) window, true);
}

int termstreambase::lines ()
{
	int r, nousado;
	getmaxyx ( (WINDOW *) w, r, nousado);
	return r;
}

int termstreambase::columns ()
{
	int r, nousado;
	getmaxyx ( (WINDOW *) w, nousado, r);
	return r;
}

int termbuf::getkey ()
{
	using namespace key;
	sync ();
	int c= wgetch ( (WINDOW *) w);
	if (c & ~ 0xFF)
		switch (c) {
		case KEY_LEFT:      c= Left;      break;
		case KEY_RIGHT:     c= Right;     break;
		case KEY_UP:        c= Up;        break;
		case KEY_DOWN:      c= Down;      break;
		case KEY_PPAGE:     c= PageUp;    break;
		case KEY_NPAGE:     c= PageDown;  break;
		case KEY_BACKSPACE: c= Backspace; break;
		case KEY_DC:        c= Delete;    break;
		case KEY_HOME:      c= Home;      break;
		case KEY_END:       c= End;       break;
		case KEY_F(1):  c= F1;  break;
		case KEY_F(2):  c= F2;  break;
		case KEY_F(3):  c= F3;  break;
		case KEY_F(4):  c= F4;  break;
		case KEY_F(5):  c= F5;  break;
		case KEY_F(6):  c= F6;  break;
		case KEY_F(7):  c= F7;  break;
		case KEY_F(8):  c= F8;  break;
		case KEY_F(9):  c= F9;  break;
		case KEY_F(10): c= F10; break;
		case KEY_F(11): c= F11; break;
		case KEY_F(12): c= F12; break;
		default: c= 0;
		}
	return c;
}

int termbuf::sync ()
{
	streamsize n= pptr () - pbase ();
	for (streamsize i= 0; i < n; ++i)
		waddch ( (WINDOW *) w, * (unsigned char *) (pbase () + i) );
	wrefresh ( (WINDOW *) w);
	pbump (-n);
	gbump (egptr () - gptr () );
	return 0;
}

int termbuf::overflow (int ch)
{
	sync ();
	if (ch != EOF) {
		waddch ( (WINDOW *) w, ch);
		wrefresh ( (WINDOW *) w);
	}
	return 0;
}

int termbuf::underflow ()
{
	return 0;
}

// Fin de termstream.cpp
